@@ -12,12 +12,9 @@ exec tclsh "$0" ${1+"$@"}
 #
 #----------------------------------------------------------------------
 #
-# Copyright (c) 2004 by Kevin B. Kenny.  All rights reserved.
+# Copyright © 2004 Kevin B. Kenny.  All rights reserved.
 # See the file "license.terms" for information on usage and redistribution
 # of this file, and for a DISCLAIMER OF ALL WARRANTIES.
-#
-# RCS: @(#) $Id: installData.tcl,v 1.1.16.1 2010/05/21 12:18:17 nijtmans Exp $
-#
 #----------------------------------------------------------------------
 
 proc copyDir {d1 d2} {
@@ -35,7 +32,7 @@ proc copyDir {d1 d2} {
 	} elseif {[file isfile $f]} {
 	    file copy -force $f [file join $d2 $ftail]
 	    if {$::tcl_platform(platform) eq {unix}} {
-		file attributes [file join $d2 $ftail] -permissions 0644
+		file attributes [file join $d2 $ftail] -permissions 0o644
 	    } else {
 		file attributes [file join $d2 $ftail] -readonly 1
 	    }
@@ -43,7 +40,7 @@ proc copyDir {d1 d2} {
     }
 
     if {$::tcl_platform(platform) eq {unix}} {
-	file attributes $d2 -permissions 0755
+	file attributes $d2 -permissions 0o755
     } else {
 	file attributes $d2 -readonly 1
     }
